#define _GNU_SOURCE /* required for RTLD_NEXT on Linux */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <dlfcn.h>
#include "mylib.h"

#define MY_FILE		"./myshared.so"
#define MY_SYM_INT	"myint"
#define MY_SYM_FUNC	"myfunc"
#define MY_SYM_Z        "zlibVersion"

int mylib_test(void)
{
  void *h;
  uint32_t *myint;
  uint64_t (*myfunc)(double);
  uint64_t (*myfunc_so_handle)(void);
  uint64_t (*myfunc_so_special)(void);
  uint64_t result;

  /* dlopen shared object */
  h = dlopen(MY_FILE, RTLD_LAZY);
  if (h == NULL) {
    printf("dlopen() failed: %s\n", dlerror());
    exit(1);
  }

  /* access variable defined in shared object */
  myint = (uint32_t *) dlsym(h, MY_SYM_INT);

  if (myint == NULL) {
    printf("dlsym() failed: %s\n", dlerror());
  }
  else {
    printf("%s = 0x%08x (%u)\n", MY_SYM_INT, *myint, *myint);
  }

  /* access function defined in shared object */
  myfunc = dlsym(h, MY_SYM_FUNC);

  if (myfunc == NULL) {
    printf("dlsym() failed: %s\n", dlerror());
  }
  else {
    result = (*myfunc)(3.14159);
  }

  /* access function defined in library linked into shared
     object using dlopen() handle */
  myfunc_so_handle = dlsym(h, MY_SYM_Z);
  if (myfunc_so_handle == NULL) {
    printf("dlsym() using dlopen handle failed: %s\n", dlerror());
  }
  else {
    printf("dlsym() using dlopen handle succeeded\n");
  }

  /* access function defined in library linked into shared
     object using RTLD_NEXT */
  myfunc_so_special = dlsym(RTLD_NEXT, MY_SYM_Z);
  if (myfunc_so_special == NULL) {
    printf("dlsym() using RTLD_NEXT failed: %s\n", dlerror());
  }
  else {
    printf("dlsym() using RTLD_NEXT succeeded\n");
  }

  dlclose(h);
  return 0;
}

